#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#include "global.h"
#include "m_timer.h"

// STOP, CLK, CLK/8, CLK/64, CLK/256, CLK/1024
unsigned short __attribute__ ((progmem)) TimerPrescaleFactor[] = {0,1,8,64,256,1024};
// the prescale division values stored in order of timer control register index
// STOP, CLK, CLK/8, CLK/32, CLK/64, CLK/128, CLK/256, CLK/1024
unsigned short __attribute__ ((progmem)) TimerRTCPrescaleFactor[] = {0,1,8,32,64,128,256,1024};

// delay for a minimum of <us> microseconds 
// the time resolution is dependent on the time the loop takes 
// e.g. with 4Mhz and 5 cycles per loop, the resolution is 1.25 us 
void delay_us(unsigned short time_us) 
{
	unsigned short delay_loops;
	register unsigned short i;

	delay_loops = (time_us+3)/5*CYCLES_PER_US; // +3 for rounding up (dirty) 

	// one loop takes 5 cpu cycles 
	for (i=0; i < delay_loops; i++) {};
}

void timer0Clear() {
	TCNT0 = 0;
}
void timer0SetCompare(u08 comp) {
	OCR0A = comp;
}

void timer0InitCompare() {
	GTCCR = 0x00;
	TCCR0A = (0<<COM0A1) //toggle
		   | (1<<COM0A0)
		   | (1<<WGM01)	//CTC
		   | (0<<WGM00);
	TCCR0B = (0<<CS02)
		   | (1<<CS01)  //div by 64
		   | (1<<CS00);
	TCNT0  = 0x00;
//	sbi(TIMSK, TOIE0);						// enable TCNT0 overflow interrupt
}

void timer0CCPOn() {
	TCCR0A = (0<<COM0A1) //toggle
		   | (1<<COM0A0)
		   | (1<<WGM01)	//CTC
		   | (0<<WGM00);
}
void timer0CCPOff() {
	TCCR0A = (0<<COM0A1) //normal port
		   | (0<<COM0A0)
		   | (1<<WGM01)	//CTC
		   | (0<<WGM00);
}
