/**************************************************************************
	Copyright, Zoltan Topor (Topi) in 2008
	Website: http://www.hobbielektronika.hu/
	To this code, the HE #1A PCB is needed. 
	This code is distributed under the GNU Public License
		which can be found at http://www.gnu.org/licenses/gpl.txt
	Target MCU	: ATtiny45
	Version		: 1.0A
***************************************************************************/

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#include "global.h"						//Minden fontos
#include "m_extint.c"					//Kls breszt interrupt
#include "m_timer.c"					//Idzt cuccai
#include <util/delay.h>
#include "melodies.c" 					//Muzsikk betlt

volatile u08 gOctave = 0;				//globl oktv
volatile u08 gTempo;					//globl tempo
volatile u08 gActMelody;				//melyik dallam
volatile u08 ChrIndex = 0x00; 			//hol tart a zenben

u08 PlayMelody(u08 Melody_ID); 
void PlayNote(u08 note, u08 octave, u16 duration); 
u08 GetByteFromData(u08 index); 


//----------------------------------------------------------------------------- 
void m_delay_ms(u16 del) {
//----------------------------------------------------------------------------- 
	for(u16 i=0;i<del;i++) _delay_ms(1);
}

//----------------------------------------------------------------------------- 
int main(void)
//----------------------------------------------------------------------------- 
{
    set_sleep_mode(SLEEP_MODE_PWR_DOWN);	//Power-down energiatakarkos md
	extint0Configure(EXTINT_EDGE_FALLING);	//Lefut lre triggereljen
	timer0InitCompare();					//Idzt inicializls

	gActMelody = 0;							//Els bekapcsnl 0-s dalt

	DDRB = (1<<PINB0);						//az az egy kimenet
	PORTB = (1<<PINB2);						//bemeneten pullup

	for(;;)
	{
		if(PlayMelody(gActMelody)) {
			do {								//Megvizsgljuk jogos-e az breszt...
				sei();							//SEt Interrupts (Globl interrupt bekapcs)
				sleep_mode();					//Hajr, menjnk aludni
				m_delay_ms(100);				//Megvizsgljuk a jelet ksbb
			} while ((PINB & (1<<PINB2)));		//Addig amg le nincs nyomva

			gActMelody++;					//Felbredtnk?! Akkor a kv. muzsika
		}
	}
	return 0;
}

//----------------------------------------------------------------------------- 
u08 PlayMelody(u08 Melody_ID)
//----------------------------------------------------------------------------- 
{ 

  volatile u16 duration = 0;
  volatile u08 octave = 0;
  volatile u16 note = 0;
  volatile u08 van_nyujt = 0;
  volatile u08 ch;

	gActMelody = Melody_ID;
	// Vals dallam index?
	if (Melody_ID >= LAST_MELODY)
	{
		// Ha nem, akkor reset
		Melody_ID = 0;
		ChrIndex = 0x00;
		gActMelody = 0;
	}

	// Egyesvel vgig...
	if(ChrIndex == 0) {
	  //ha most indul
	  gTempo = GetByteFromData(ChrIndex++);
	}

	ch = GetByteFromData(ChrIndex++);

	duration = ch & 0b00111111;
	van_nyujt = ch & 0b10000000;

	ch = GetByteFromData(ChrIndex++);
	note = pgm_read_byte(Notes+(ch & 0b00001111));

	octave = (ch & 0b11100000)>>5;

    PlayNote(note, octave, duration); 
	
    // Vge a muzsiknak
    if (GetByteFromData(ChrIndex) == 0) 
    { 
      ChrIndex = 0x00; 
      return 1;//igen vgzett
    }
	return 0;//nem, nem vgzett
} 

//----------------------------------------------------------------------------- 
void PlayNote(u08 note, u08 octave, u16 duration) 
//----------------------------------------------------------------------------- 
{ 
  u16 adur;
  // Oktv szmols
  switch (octave) 
  { 
    case 4 : break; 
    case 5 : note = note>>1; break; 
    case 6 : note = note>>2; break; 
    case 7 : note = note>>4; break; 
  } 
  adur = (60000/gTempo/duration);			//id kiszmolsa

  if (note==0) m_delay_ms(adur); else {
   timer0Clear();							//Szmll trls
   timer0SetCompare(note);					//j komp. szint - j freki
   timer0CCPOn();							//Jel bekapcs
   m_delay_ms(adur);						//Vrok hang hossznyit
   timer0CCPOff();							//Jel kikapcs
  }
}  

//----------------------------------------------------------------------------- 
u08 GetByteFromData(u08 ind) 
//----------------------------------------------------------------------------- 
{ 
  //tmegesen kivesszk az index szerinti bjtot a _PROG_memribl
  switch (gActMelody) 
  { 
    case 0: return  pgm_read_byte(Melody001+ind); break; 
    case 1: return  pgm_read_byte(Melody002+ind); break; 
    case 2: return  pgm_read_byte(Melody003+ind); break; 
    case 3: return  pgm_read_byte(Melody004+ind); break; 
    case 4: return  pgm_read_byte(Melody005+ind); break; 
    case 5: return  pgm_read_byte(Melody006+ind); break; 
    case 6: return  pgm_read_byte(Melody007+ind); break; 
    case 7: return  pgm_read_byte(Melody008+ind); break; 
    case 8: return  pgm_read_byte(Melody009+ind); break; 
    case 9: return  pgm_read_byte(Melody010+ind); break; 
    case 10: return pgm_read_byte(Melody011+ind); break; 
    case 11: return pgm_read_byte(Melody012+ind); break; 
    case 12: return pgm_read_byte(Melody013+ind); break; 
    case 13: return pgm_read_byte(Melody014+ind); break; 
    case 14: return pgm_read_byte(Melody015+ind); break; 
    case 15: return pgm_read_byte(Melody016+ind); break; 
    case 16: return pgm_read_byte(Melody017+ind); break; 
    case 17: return pgm_read_byte(Melody018+ind); break; 
    case 18: return pgm_read_byte(Melody019+ind); break; 
    case 19: return pgm_read_byte(Melody020+ind); break; 
    default: break; 
  } 
  return 0;
} 


